import json
import textwrap

import oyaml as yaml
from cloudmesh.common.FlatDict import flatten
from cloudmesh.common.util import convert_from_unicode
from tabulate import tabulate as tabulate_printer


# noinspection PyPep8
class Printer:
    """
    formats supported dict, yaml, json

    set table.format=table
    cms flavor list
    +-----------+---------+-------+--------+
    | Name      |   VCPUS |   RAM |   Disk |
    |-----------+---------+-------+--------|
    | m1.tiny   |       1 |   512 |      1 |
    | m1.small  |       1 |  2048 |     20 |
    | m1.medium |       2 |  4096 |     40 |
    | m1.large  |       4 |  8192 |     40 |
    | m1.xlarge |       8 | 16384 |     40 |
    +-----------+---------+-------+--------+

    set table.format=grid
    cms flavor list
    ╒═══════════╤═════════╤═══════╤════════╕
    │ Name      │   VCPUS │   RAM │   Disk │
    ╞═══════════╪═════════╪═══════╪════════╡
    │ m1.tiny   │       1 │   512 │      1 │
    ├───────────┼─────────┼───────┼────────┤
    │ m1.small  │       1 │  2048 │     20 │
    ├───────────┼─────────┼───────┼────────┤
    │ m1.medium │       2 │  4096 │     40 │
    ├───────────┼─────────┼───────┼────────┤
    │ m1.large  │       4 │  8192 │     40 │
    ├───────────┼─────────┼───────┼────────┤
    │ m1.xlarge │       8 │ 16384 │     40 │
    ╘═══════════╧═════════╧═══════╧════════╛

    """

    @staticmethod
    def default():
        return "psql"

    @staticmethod
    def select(results, order=None, width=None):
        """
        selects field in the order as defined in order
        If order is non, all fileds will be taken

        :param results: a flat dict
        :param order: the order of the fields
        :param width:
        :return: a list of dicts
        """
        if order is None:
            columns = len(results[0])
        else:
            columns = len(order)

        if type(width) == int:
            _width = [width] * columns
        elif type(width) == list:
            _width = width
        else:
            _width = [40] * columns

        if order is None or results is None:
            return results
        _results = []
        for result in results:
            if width is None:
                _results.append([result[key] for key in order])
            else:
                entry = []
                i = 0
                for key in order:
                    if _width[i]:
                        try:
                            field = result[key]
                            if type(field) == list:
                                field = ", ".join(field)
                            entry.append("\n".join(
                                textwrap.wrap(str(field), _width[i])))
                        except:  # noqa: E722
                            entry.append("")
                    else:
                        entry.append(result[key])
                    i = i + 1
                _results.append(entry)
        return _results

    @staticmethod
    def write(results,
              order=None,
              header=None,
              output='table',
              width=None,
              humanize=False,
              max_width=None,
              sep='.'):

        width = width or max_width  # maxwidth is deprecated

        if header:
            header = list(header)
        elif order:
            header = list(order)

        if output == 'table':
            output = Printer.default()

        #
        # Make sure that we have a list of dicts
        #
        if type(results) == dict:
            _results = Printer._to_tabulate(results)
        else:
            _results = list(results)

        if output == "json":
            return json.dumps(_results,
                              # sort_keys=sort_keys,
                              indent=4)
        elif output == "yaml":
            return yaml.dump(convert_from_unicode(_results),
                             default_flow_style=False)
        elif output == "dict":
            return _results
        elif output == 'csv':
            flat = flatten(_results, sep=sep)

            return Printer.csv(flat,
                               order=order,
                               header=header,
                               humanize=humanize,
                               sort_keys=True)

        _results = Printer.select(_results,
                                  order=order,
                                  width=width)

        if output in ['flat', 'html']:

            if order is not None:

                flat = []
                for element in results:
                    _element = {}
                    for attribute in element:
                        if attribute in order:
                            _element[attribute] = element[attribute]
                    flat.append(_element)

                _results = flat

        if output in ['flat']:
            return (_results)

        if header:
            return tabulate_printer(_results, tablefmt=output, headers=header)
        else:
            return tabulate_printer(_results, tablefmt=output)

    @staticmethod
    def _to_tabulate(d):
        """

        :param d: dict of dicts
        :return: list of dicts with the key as name
        """
        results = []
        for key in d:
            results.append(d[key])

        return results

    @staticmethod
    def flatwrite(table,
                  order=None,
                  header=None,
                  output="table",
                  sort_keys=True,
                  show_none="",
                  humanize=None,
                  sep=".",
                  max_width=48,  # deprecated use width
                  width=48,
                  ):
        """
        writes the information given in the table
        :param table: the table of values
        :param order: the order of the columns
        :param header: the header for the columns
        :param output: the format (default is table, values are raw, csv, json, yaml, dict
        :param sort_keys: if true the table is sorted
        :param show_none: passed along to the list or dict printer
        :param sep: uses sep as the separator for csv printer
        :param width: maximum width for a cell
        :type width: int
        :param max_width: deprecated use width
        :type max_width: int
        :return:
        """
        width = width or max_width  # max_width is deprecated. thi is used for those still using it

        flat = flatten(table, sep=sep)

        return Printer.write(flat,
                             order=order,
                             header=header,
                             output=output,
                             width=width
                             )
        """
        return Printer.write(flat,
                             sort_keys=sort_keys,
                             order=order,
                             header=header,
                             output=output,
                             humanize=humanize,
                             width=width
                             )
        """

    @classmethod
    def attribute(cls, d,
                  header=None,
                  order=None,
                  sort_keys=True,
                  humanize=None,
                  output="table",
                  width=70):
        """
        prints a attribute/key value table

        :param d: A a dict with dicts of the same type.
                  Each key will be a column
        :param order: The order in which the columns are printed.
                       The order is specified by the key names of the dict.
        :param header:  The Header of each of the columns
        :type header:   A list of string
        :param sort_keys:   Key(s) of the dict to be used for sorting.
                             This specify the column(s) in the table for sorting.
        :type sort_keys:    string or a tuple of string (for sorting with multiple columns)
        :param output: the output format table, csv, dict, json
        """

        if header is None:
            header = ["Attribute", "Value"]
        x = []
        if order is not None:
            sorted_list = order
        else:
            sorted_list = list(d)
        if sort_keys:
            sorted_list = sorted(d)

        for key in sorted_list:
            if type(d[key]) == dict:
                values = d[key]
                x.append([key, "+"])
                for e in values:
                    x.append(["  - {}: {}".format(e, values[e])])
            elif type(d[key]) == list:
                values = list(d[key])
                x.append([key, "+"])
                for e in values:
                    x.append(["  -", e])
            else:
                x.append([key, str(d[key]) or ""])

        if output == 'table':
            output = Printer.default()

        return tabulate_printer(x, tablefmt=output, headers=header)

    @classmethod
    def csv(cls, d,
            order=None,
            header=None,
            humanize=None,
            sort_keys=True):
        """
        prints a table in csv format

        :param d: A a dict with dicts of the same type.
        :type d: dict
        :param order: The order in which the columns are printed.
                      The order is specified by the key names of the dict.
        :type order:
        :param header: The Header of each of the columns
        :type header: list or tuple of field names
        :param sort_keys: TODO - not yet implemented
        :type sort_keys: bool
        :return: a string representing the table in csv format
        """

        if order is None:
            first_element = list(d)[0]

        def _keys():
            return list(first_element.keys())

        # noinspection PyBroadException
        def _get(element, key):
            try:
                tmp = str(d[element][key])
            except:  # noqa: E722
                tmp = ' '
            return tmp

        if d is None or d == {}:
            return None

        if order is None:
            order = _keys()

        if header is None and order is not None:
            header = order
        elif header is None:
            header = _keys()

        table = ""
        content = []
        for attribute in order:
            content.append(attribute)

        table = table + ",".join([str(e) for e in content]) + "\n"

        for job in d:
            content = []
            for attribute in order:
                try:
                    # if attribute in humanize:
                    #    value = parser.parse(d[job][attribute])
                    #    content.append(naturaltime(value))
                    #    pass
                    # else:
                    content.append(job[attribute])

                except Exception as e:
                    content.append(e)
            table = table + ",".join([str(e) for e in content]) + "\n"
        return table



import os
from cloudmesh.common.util import path_expand


class EncryptFile(object):
    def __init__(self, file_in, file_out, debug=False):
        self.data = {
            'file': file_in,
            'secret': file_out,
            'pem': path_expand('~/.ssh/id_rsa.pub.pem'),
            'key': path_expand(' ~/.ssh/id_rsa')
        }
        self.debug = debug

    def _execute(self, command):
        if self.debug:
            print(command)
        os.system(command)

    def pem_create(self):
        command = path_expand("openssl rsa -in {key} -pubout  > {pem}".format(**self.data))
        self._execute(command)

    def pem_cat(self):
        command = path_expand("cat {pem}".format(**self.data))
        self._execute(command)

    def encrypt(self):
        # encrypt the file into secret.txt
        print(self.data)
        command = path_expand(
            "openssl rsautl -encrypt -pubin -inkey {pem} -in {file} -out {secret}".format(**self.data))
        self._execute(command)

    def decrypt(self, filename=None):
        if filename is not None:
            self.data['secret'] = filename

        command = path_expand("openssl rsautl -decrypt -inkey {key} -in {secret}".format(**self.data))
        self._execute(command)


if __name__ == "__main__":

    for filename in ['file.txt', 'secret.txt']:
        try:
            os.remove(filename)
        except FileNotFoundError:
            pass

    # Creating a file with data

    with open("file.txt", "w") as f:
        f.write("Big Data is here.")

    e = EncryptFile('file.txt', 'secret.txt')
    e.encrypt()
    e.decrypt()



import inspect
import os
from pprint import pprint

import yaml
from cloudmesh.common.StopWatch import StopWatch
from cloudmesh.common.util import path_expand
from cloudmesh.common.variables import Variables


# pylint: disable=C0103
# noinspection PyPep8Naming
class Benchmark(object):

    @staticmethod
    def debug():
        """
        sets the cms shell variables

          trace = True
          debug = True
          verbose = 10

        """
        variables = Variables()
        variables["trace"] = True
        variables["debug"] = True
        variables["verbose"] = 10

    @staticmethod
    def name(with_class=False):
        """
        name of the calling method

        :return: the name
        """
        frame = inspect.getouterframes(inspect.currentframe())
        method = frame[2][3]

        # pprint (frame[2])

        if with_class:
            classname = os.path.basename(frame[2].filename).replace(".py", "")
            method = classname + "/" + method
        return method

    @staticmethod
    def Start(status=True):
        """
        starts a timer while using the name of the calling method
        """
        StopWatch.start(Benchmark.name(with_class=True))

    @staticmethod
    def Status(value=True):
        """
        starts a timer while using the name of the calling method
        """
        StopWatch.status(Benchmark.name(with_class=True), value)

    @staticmethod
    def Stop():
        """
        stops a timer while using the name of the calling method
        """
        StopWatch.stop(Benchmark.name(with_class=True))
        StopWatch.status(Benchmark.name(with_class=True), True)

    @staticmethod
    def print(sysinfo=True,
              csv=True,
              tag=None,
              node=None,
              user=None, ):
        """
        prints the benchmark information with all timers
        """
        StopWatch.start("benchmark_start_stop")
        StopWatch.stop("benchmark_start_stop")
        StopWatch.benchmark(sysinfo=sysinfo, csv=csv, tag=tag, user=user, node=node)

    @staticmethod
    def yaml(path, n):
        """
        creates a cloudmesh service yaml test file

        Example: BenchmarkFiles.yaml("./t.yaml", 10)

        :param path: the path
        :param n: number of services
        :return:
        """
        cm = {
            "cloudmesh": {}
        }
        for i in range(0, n):
            cm["cloudmesh"][f"service{i}"] = {
                "attribute": f"service{i}"
            }
        pprint(cm)
        location = path_expand(path)

        with open(location, 'w') as yaml_file:
            yaml.dump(cm, yaml_file, default_flow_style=False)

    # noinspection SpellCheckingInspection
    @staticmethod
    def file(path, n):
        """
        create a file of given size in MB, the MB here is in binary not SI
        units.
        e.g. 1,048,576 Bytes

        Example: s = BenchmarkFiles.size("./sise.txt", 2)
                 print(s)

        :param path: the filename and path
        :type path: string
        :param n: the size in binary MB
        :type n: integer
        :return: size in MB
        :rtype: float
        """
        location = path_expand(path)
        size = 1048576 * n  # size in bytes
        with open(path, "wb") as f:
            f.write(os.urandom(size))

        s = os.path.getsize(location)
        # try:
        #    os.system(f"ls -lhs {location}")
        #    os.system(f"du -h {location}")
        # except:
        #    pass

        return s / 1048576.0



"""
A simple framework to handle error messages
"""
import sys
import traceback
from cloudmesh.common.console import Console


#
# TODO: this class seems to replicate some portions of what Console does
#
class Error(object):
    """
    A class to print error messages
    """

    #
    # TODO: this should probably use Console so we can print in color
    #
    @classmethod
    def msg(cls, error=None, debug=True, trace=True):
        """
        prints the error message

        :param error: the error message
        :param debug: only prints it if debug is set to true
        :param trace: if true prints the trace
        :return:
        """
        if debug and error is not None:
            print(error)
        if debug and trace:
            print(traceback.format_exc())

    @classmethod
    def traceback(cls, error=None, debug=True, trace=True):
        """
        prints the trace

        :param error: a message preceding the trace
        :param debug: prints it if debug is set to true
        :param trace:
        :return:
        """
        if debug and trace:
            Error.msg(error=error, debug=debug, trace=trace)

    @classmethod
    def info(cls, msg, debug=True):
        """
        prints an info msg.

        :param msg: the message
        :return:
        """
        if debug:
            Console.info(msg)

    @classmethod
    def warning(cls, msg, debug=True):
        """
        prints a warning message.

        :param msg:
        :return:
        """
        if debug:
            Console.warning(msg)

    @classmethod
    def debug(cls, msg, debug=True):
        """
        prints a debug message.

        :param msg: the message
        :return:
        """
        if debug:
            Console.msg(msg)

    @classmethod
    def exit(cls, msg):
        """
        call a system exit

        :param msg:
        :return:
        """
        Console.error(msg)
        sys.exit()



"""
A class to call a TODO message. Typically it will just raise na exception.
However when using the TODO it allows to list the TODO in editors such as
pycharm to easier find them.
"""


class TODO(object):
    """
    class to raise an exception for code that has not yet been implemented.

    import cloudmesh.common.todo
    TODO.implement()
    """

    @classmethod
    def implement(cls, msg="Please implement"):
        """
        Raises an exception as not implemented
        :param msg: the message to print
        :return:
        """
        print(msg)
        raise NotImplementedError(msg)



#!/usr/bin/env python
# ----------------------------------------------------------------------- #
# Copyright 2017, Gregor von Laszewski, Indiana University                #
#                                                                         #
# Licensed under the Apache License, Version 2.0 (the "License"); you may #
# not use this file except in compliance with the License. You may obtain #
# a copy of the License at                                                #
#                                                                         #
# http://www.apache.org/licenses/LICENSE-2.0                              #
#                                                                         #
# Unless required by applicable law or agreed to in writing, software     #
# distributed under the License is distributed on an "AS IS" BASIS,       #
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.#
# See the License for the specific language governing permissions and     #
# limitations under the License.                                          #
# ------------------------------------------------------------------------#

import io
import sys

from setuptools import find_packages, setup

def check_python():
    if not sys.version_info.major == 3 and \
        sys.version_info.minor >= 7:
        print("Python 3.7 or higher is required.")

        print("You are using Python {}.{}."
              .format(sys.version_info.major,
                      sys.version_info.minor))

        sys.exit(1)

check_python()

def readfile(filename):
    with io.open(filename, encoding="utf-8") as stream:
        return stream.read()


# requiers = readfile ('requirements.txt')

# pytz==2016.10

requiers = """
psutil
pytz
python-hostlist
simplejson
oyaml
colorama
humanize
python-dateutil
tabulate
requests
pyfiglet
tqdm
""".splitlines()

version = readfile("VERSION").strip()

with open('README.md') as f:
    long_description = f.read()

NAME = "cloudmesh-common"
DESCRIPTION = "A set of useful APIs for cloudmesh"
AUTHOR = "Gregor von Laszewski"
AUTHOR_EMAIL = "laszewski@gmail.com"
URL = "https://github.com/cloudmesh/cloudmesh-common"

setup(
    name=NAME,
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type="text/markdown",
    version=version,
    license="Apache 2.0",
    url=URL,
    packages=find_packages(exclude=("tests")),
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Environment :: Other Environment",
        "Environment :: Plugins",
        "Intended Audience :: Information Technology",
        "Intended Audience :: Developers",
        "Intended Audience :: Education",
        "Intended Audience :: Science/Research",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: Apache Software License",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: POSIX :: Linux",
        "Operating System :: Microsoft :: Windows :: Windows 10",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Topic :: Internet",
        "Topic :: Scientific/Engineering",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: User Interfaces",
        "Topic :: System",
        "Topic :: System :: Distributed Computing",
        "Topic :: System :: Shells",
        "Topic :: Utilities",
    ],
    install_requires=requiers,
    tests_require=[
        "flake8",
        "coverage",
    ],
    zip_safe=False,
    namespace_packages=['cloudmesh'],
)



"""
Printing messages in a console
"""
import textwrap
import traceback
import colorama
from colorama import Fore, Back, Style
import os

# from cloudmesh.common.variables import Variables

colorama.init()


def indent(text, indent=2, width=128):
    """
    indents the given text by the indent specified and wrapping to the given width

    :param text: the text to print
    :param indent: indent characters
    :param width: the width of the text
    :return:
    """
    return "\n".join(
        textwrap.wrap(text,
                      width=width,
                      initial_indent=" " * indent,
                      subsequent_indent=" " * indent))


class Console(object):
    """
    A simple way to print in a console terminal in color. Instead of using
    simply the print statement you can use special methods to indicate
    warnings, errors, ok and regular messages.

    Example Usage::

        Console.warning("Warning")
        Console.error("Error")
        Console.info("Info")
        Console.msg("msg")
        Console.ok("Success")

    One can switch the color mode off with::

        Console.color = False
        Console.error("Error")

    The color will be switched on by default.
    """

    color = True
    debug = True

    theme_color = {
        'HEADER': Fore.MAGENTA,
        'BLACK': Fore.BLACK,
        'CYAN': Fore.CYAN,
        'WHITE': Fore.WHITE,
        'BLUE': Fore.BLUE,
        'OKBLUE': Fore.BLUE,
        'OKGREEN': Fore.GREEN,
        'GREEN': Fore.GREEN,
        'FAIL': Fore.RED,
        'WARNING': Fore.MAGENTA,
        'RED': Fore.RED,
        'ENDC': Style.RESET_ALL,
        'BOLD': Style.BRIGHT,
        'NORMAL': Style.NORMAL
        # 'ENDC': '\033[0m',
        # 'BOLD': "\033[1m",
    }

    theme_bw = {
        'HEADER': '',
        'BLACK': '',
        'CYAN': '',
        'WHITE': '',
        'BLUE': '',
        'OKBLUE': '',
        'OKGREEN': '',
        'GREEN': '',
        'FAIL': '',
        'WARNING': '',
        'RED': '',
        'ENDC': '',
        'BOLD': "",
        'NORMAL': ""
    }

    theme = theme_color

    try:
        size = os.get_terminal_size()
        columns = size.columns
        lines = size.lines
    except:  # noqa: E722
        columns = 79
        lines = 24

    def line(self, c="="):
        try:
            size = os.get_terminal_size()
            columns = size.columns
        except:  # noqa: E722
            columns = 79

        # lines = size.lines

        print(columns * c)

    # noinspection PyPep8Naming
    def red(msg):
        print(Fore.RED + msg + Style.RESET_ALL)

    # noinspection PyPep8Naming
    def green(msg):
        print(Fore.GREEN + msg + Style.RESET_ALL)

    # noinspection PyPep8Naming
    def blue(msg):
        print(Fore.BLUE + msg + Style.RESET_ALL)

    @staticmethod
    def init():
        """
        initializes the Console
        """
        colorama.init()

    @staticmethod
    def terminate():
        """
        terminates the program
        """
        os._exit(1)

    @classmethod
    def set_debug(cls, on=True):
        """
        sets debugging on or of

        :param on: if on debugging is set
        :return:
        """
        cls.debug = on

    @staticmethod
    def set_theme(color=True):
        """
        defines if the console messages are printed in color

        :param color: if True its printed in color
        :return:
        """
        if color:
            Console.theme = Console.theme_color
        else:
            Console.theme = Console.theme_bw
        Console.color = color

    @staticmethod
    def get(name):
        """
        returns the default theme for printing console messages

        :param name: the name of the theme
        :return:
        """
        if name in Console.theme:
            return Console.theme[name]
        else:
            return Console.theme['BLACK']

    @staticmethod
    def txt_msg(message, width=79):
        """
        prints a message to the screen

        :param message: the message to print
        :param width: teh width of the line
        :return:
        """
        return textwrap.fill(message, width=width)

    @staticmethod
    def msg(*message):
        """
        prints a message

        :param message: the message to print
        :return:
        """
        str = " ".join(message)
        print(str)

    @staticmethod
    def bullets(elements):
        """
        prints elemnets of a list as bullet list

        :param elements: the list
        """
        for name in elements:
            print("*", name)

    @classmethod
    def error(cls, message, prefix=True, traceflag=False):
        """
        prints an error message

        :param message: the message
        :param prefix: a prefix for the message
        :param traceflag: if true the stack trace is retrieved and printed
        :return:
        """
        # print (message, prefix)
        # variables = Variables()
        # traceflag = traceflag or variables['trace']

        message = message or ""
        if prefix:
            text = "ERROR: "
        else:
            text = ""
        if cls.color:
            cls.cprint('FAIL', text, str(message))
        else:
            print(cls.txt_msg(text + str(message)))

        if traceflag and cls.debug:
            trace = traceback.format_exc().strip()
            if trace:
                print()
                print("Trace:")
                print("\n    ".join(str(trace).splitlines()))
                print()

    @staticmethod
    def TODO(message, prefix=True, traceflag=True):
        """
        prints an TODO message

        :param message: the message
        :param prefix: if set to true it prints TODO: as prefix
        :param traceflag: if true the stack trace is retrieved and printed
        :return:
        """
        message = message or ""
        if prefix:
            text = "TODO: "
        else:
            text = ""
        if Console.color:
            Console.cprint('FAIL', text, str(message))
        else:
            print(Console.msg(text + str(message)))

        trace = traceback.format_exc().strip()

        if traceflag and trace != "None":
            print()
            print("\n".join(str(trace).splitlines()))
            print()

    @staticmethod
    def debug_msg(message):
        """
        print a debug message

        :param message: the message
        :return:
        """
        message = message or ""
        if Console.color:
            Console.cprint('RED', 'DEBUG: ', message)
        else:
            print(Console.msg('DEBUG: ' + message))

    @staticmethod
    def info(message):
        """
        prints an informational message

        :param message: the message
        :return:
        """
        message = message or ""
        if Console.color:
            Console.cprint('OKBLUE', "INFO: ", message)
        else:
            print(Console.msg("INFO: " + message))

    @staticmethod
    def warning(message):
        """
        prints a warning

        :param message: the message
        :return:
        """
        message = message or ""
        if Console.color:
            Console.cprint('WARNING', "WARNING: ", message)
        else:
            print(Console.msg("WARNING: " + message))

    @staticmethod
    def ok(message):
        """
        prints an ok message

        :param message: the message<
        :return:
        """
        message = message or ""
        if Console.color:
            Console.cprint('OKGREEN', "", message)
        else:
            print(Console.msg(message))

    @staticmethod
    def cprint(color="BLUE", prefix="", message=""):
        """
        prints a message in a given color

        :param color: the color as defined in the theme
        :param prefix: the prefix (a string)
        :param message: the message
        :return:
        """
        message = message or ""
        prefix = prefix or ""

        print(Console.theme_color[color] + prefix + message + Console.theme_color['ENDC'])

    @staticmethod
    def text(color='RED', prefix=None, message=None):
        """
        returns a message in a given color

        :param color: the color as defined in the theme
        :param prefix: the prefix (a string)
        :param message: the message
        :return:
        """
        message = message or ""
        prefix = prefix or ""
        return (Console.theme[color] + prefix + message + Console.theme['ENDC'])


#
# Example
#


if __name__ == "__main__":
    print(Console.color)

    print(Console.theme)

    Console.line()
    Console.warning("Warning")
    Console.error("Error")
    Console.info("Info")
    Console.msg("msg")
    Console.ok("Ok")

    Console.color = False
    print(Console.color)
    Console.error("Error")

    print(Fore.RED + 'some red text')
    print(Back.GREEN + 'and with a green background')
    print(Style.DIM + 'and in dim text')
    print(Fore.RESET + Back.RESET + Style.RESET_ALL)
    print('back to normal now')
    Console.line()



# This file was copied from https://github.com/jdjebi/winshlex
#
import sys
import re


def split(s, platform='this'):
    """Multi-platform variant of shlex.split() for command-line splitting.
    For use with subprocess, for argv injection etc. Using fast REGEX.

    platform: 'this' = auto from current platform;
              1 = POSIX;
              0 = Windows/CMD
              (other values reserved)
    """
    if platform == 'this':
        platform = (sys.platform != 'win32')
    if platform == 1:
        RE_CMD_LEX = r'''"((?:\\["\\]|[^"])*)"|'([^']*)'|(\\.)|(&&?|\|\|?|\d?\>|[<])|([^\s'"\\&|<>]+)|(\s+)|(.)'''
    elif platform == 0:
        RE_CMD_LEX = r'''"((?:""|\\["\\]|[^"])*)"?()|(\\\\(?=\\*")|\\")|(&&?|\|\|?|\d?>|[<])|([^\s"&|<>]+)|(\s+)|(.)'''
    else:
        raise AssertionError('unkown platform %r' % platform)

    args = []
    accu = None  # collects pieces of one arg
    for qs, qss, esc, pipe, word, white, fail in re.findall(RE_CMD_LEX, s):
        if word:
            pass  # most frequent
        elif esc:
            word = esc[1]
        elif white or pipe:
            if accu is not None:
                args.append(accu)
            if pipe:
                args.append(pipe)
            accu = None
            continue
        elif fail:
            raise ValueError("invalid or incomplete shell string")
        elif qs:
            word = qs.replace('\\"', '"').replace('\\\\', '\\')
            if platform == 0:
                word = word.replace('""', '"')
        else:
            word = qss  # may be even empty; must be last

        accu = (accu or '') + word

    if accu is not None:
        args.append(accu)

    return args



import subprocess


class run(object):
    """
    A class to simply execute a common.

    Example:

        command = run("sleep 100")
        command.execute()

        ...

        command.kill()

    """

    def __init__(self, command):
        """
        Initialize the command so it can later on be executed

        :param command:
        """
        self.command = command
        self.pid = None

    def execute(self):
        """
        Execute the command

        :return:
        """
        self.proc = subprocess.Popen(self.command)
        self.pid = self.proc.pid

    def kill(self):
        """
        Kill the command

        :return:
        """
        if self.proc.poll() is None:
            print("Killing: ", self.command)
            print("Pid", self.pid)
            subprocess.Popen(['kill', f"{self.pid}"])



"""
class that specifies where we read the cloudmesh.yaml file from
"""
import os

from cloudmesh.common.Shell import Shell
from cloudmesh.common.util import path_expand
from cloudmesh.common.util import writefile
from cloudmesh.common.util import readfile
from pathlib import Path
from cloudmesh.common.console import Console


class Location:
    _shared_state = None

    def __init__(self, directory="~/.cloudmesh"):
        if not Location._shared_state:
            self.key = "CLOUDMESH_CONFIG_DIR"

            Location._shared_state = self.__dict__
            directory = path_expand(directory)
            self.directory = os.environ.get(self.key) or directory
            if not os.path.isdir(directory):
                Shell.mkdir(directory)
        else:
            self.__dict__ = Location._shared_state

    def write(self, filename, content):
        """
        Write a file to the configuration directory

        :param filename: The filename
        :param content:  The content
        :return:
        """
        path = Path(self.directory) / filename
        directory = os.path.dirname(path)
        Shell.mkdir(directory)
        writefile(filename, content)

    def read(self, filename):
        """
        Read a file from the configuration directory

        :param filename: The filename
        :return: The content
        """
        return readfile(filename)

    def file(self, filename):
        """
        The location of the config file in the cloudmesh configuration directory

        :param filename: the filenam
        """
        return Path(self.directory) / filename

    def get(self):
        return self.directory

    def set(self, directory):
        self.directory = path_expand(directory)

    def config(self):
        return self.file("cloudmesh.yaml")

    def environment(self, key):
        if key in os.environ:
            value = os.environ[key]
            self.set(value)
        else:
            Console.error(f"Config location: could not find {key}")
            return None

    def __str__(self):
        return self.directory

    def __eq__(self, other):
        return self.directory == other
